/*
 * MinimaxMonteCarlo.cpp
 *
 *  Created on: Apr 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "MinimaxMonteCarlo.h"

MinimaxMonteCarlo::MinimaxMonteCarlo(double effectiveness, double propogation) :
	RolloutMonteCarlo(effectiveness, propogation)
{ }

MinimaxMonteCarlo::~MinimaxMonteCarlo()
{ }

action_t lastBranchingFactor = 0;
double lastBaseDiff = -1;
double getBaseDiff(action_t branchingFactor) {
	//Normally trees will only have the one option
	//So we can cache that one for speed
	if(lastBranchingFactor == branchingFactor) {
		return lastBaseDiff;
	}

	// solving x^b=1-x for x, b = branchingFactor
	// No simple numerical method.
	// However, continually increasing over [0,1]
	// So just do a binary search
	double threshhold = 0.0000001;
	double low = 0;
	double high = 1;
	while(high - low > threshhold) {
		double mid = (high + low) / 2.0;
		double bGuess = log(1-mid)/log(mid);
		if(bGuess > branchingFactor) { //mid was too high
			high = mid;
		} else { //mid was too low
			low = mid;
		}
	}
	double guess = (high + low) / 2.0;

	lastBranchingFactor = branchingFactor;
	lastBaseDiff = guess;

	return guess;
}

bool MinimaxMonteCarlo::shouldPlayout(GameState& startState) {
	return RolloutMonteCarlo::shouldPlayout(startState);
	//return false;
}

void MinimaxMonteCarlo::update(GameState& state, double playoutValue) {
	SearchData& data = state.searchData;

	if(state.isTerminal()) {
		state.searchData.estimatedValue = (state.getWinner()==B) ? 1.0 : 0.0;
	} else {
		action_t children = state.countChildren();
		action_t visitedChildren = state.countVisitedChildren();
		action_t unvisitedChildren = children - visitedChildren;

		bool isMin = state.getPlayer() == A;

		double defaultDiff = getBaseDiff(children); //default diff of children!
		//if(! isMin) defaultDiff = 1 - defaultDiff;

		double estimatedValue = pow(defaultDiff, unvisitedChildren);
		for(int i=0;i<visitedChildren;i++) {
			GameState& child = state.getVisitedChild(i);
			if(child.searchData.numVisits == 0) {
				estimatedValue *= defaultDiff;
			} else if(child.isTerminal()) {
				double cev = (child.getWinner() == A) ? 0.0 : 1.0;
				estimatedValue *= isMin ? cev : 1 - cev;
			} else {
				double cev = child.searchData.estimatedValue;
				estimatedValue *= isMin ? cev : 1 - cev;
			}
		}

		data.estimatedValue = isMin ? estimatedValue : 1 - estimatedValue;
	}

	RolloutMonteCarlo::update(state, playoutValue);
}

double MinimaxMonteCarlo::evaluate(GameState& state) {
	double est = state.searchData.estimatedValue;
	if(state.getPlayer() == B) {
		return 1 - est;
	} else {
		return est;
	}
}

double MinimaxMonteCarlo::evaluateFirst(GameState& parent) {
	double est = getBaseDiff(parent.countChildren());
	if(parent.getPlayer() == A) {
		return 1 - est;
	} else {
		return est;
	}
}

double MinimaxMonteCarlo::estimateValue(GameState& state) {
	return state.searchData.estimatedValue;
}

std::string MinimaxMonteCarlo::toString() const {
	return "minimax-montecarlo";
}
