/*
 * RolloutMonteCarlo.cpp
 *
 *  Created on: Apr 15, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "RolloutMonteCarlo.h"

RolloutMonteCarlo::RolloutMonteCarlo(double simulationEffectiveness, double propogationConstant) :
	SearchAlgorithm(),
	mEffectiveness(simulationEffectiveness),
	mPropogationConstant(propogationConstant),
	mGenerator()
{ }

RolloutMonteCarlo::~RolloutMonteCarlo()
{ }

void RolloutMonteCarlo::reset() {
	//Nothing by design
}

void RolloutMonteCarlo::iterate() {
	search(mTree->getRoot());
}

double RolloutMonteCarlo::search(GameState& startState) {
	double innerSearchValue;

	if(startState.isTerminal()) {
		innerSearchValue = (startState.getWinner() == A) ? 0.0 : 1.0;
	} else if(shouldPlayout(startState)) {
		innerSearchValue = simulate(startState);
	} else {
		GameState& nextState = startState.getChild(select(startState));

		double playout = search(nextState);
		double reward = estimateValue(nextState);

		innerSearchValue =
				mPropogationConstant * playout +
				(1 - mPropogationConstant) * reward;
	}

	update(startState, innerSearchValue);

	return innerSearchValue;
}

bool RolloutMonteCarlo::shouldPlayout(GameState& startState) {
	return startState.searchData.numVisits < 1;
	//return startState.getDepth() >= 2;
	//return false;
}

player_t RolloutMonteCarlo::simulate(GameState& startState) {
	//We want to return the true winner with probability mEffectiveness
	double r = mGenerator.nextReal();
	if(r < mEffectiveness) {
		return startState.getWinner();
	} else {
		//Now we want to estimate it based on difficulty
		//See MQP Theorem 2 for the details of why this works
		double difficulty = startState.getDifficulty();
		double childWinProb = 1.0 / (1.0 + exp(-difficulty));
		double b = startState.countChildren();
		double a = (b-1.0)/b;
		double binv = 1.0/b;
		double xa = childWinProb*a;

		double monteCarloWinProb =
				(xa * (1 + xa + binv)) /
				(2 * (xa * (xa + binv - 1) + 1 - binv));

		double randomReal = mGenerator.nextReal();

		//Higher monteCarloWinProb means better odds for B
		if(monteCarloWinProb >= randomReal) {
			return B;
		} else {
			return A;
		}
	}
}

action_t RolloutMonteCarlo::select(GameState& startState) {
	action_t numVisitedChildren = startState.countVisitedChildren();
	action_t bestAction = 0;
	double bestValue = -DBL_MAX;

	double firstPlayValue = evaluateFirst(startState);
	double cValue;

	for(action_t c = 0; c < numVisitedChildren; c++) {
		GameState& child = startState.getVisitedChild(c);
		if(child.searchData.numVisits == 0) {
			cValue = firstPlayValue;
		} else {
			cValue = evaluate(child);
		}

		if(cValue > bestValue) {
			bestValue = cValue;
			bestAction = c;
		}
	}

	if(	numVisitedChildren == 0 ||
		(	cValue < firstPlayValue &&
			numVisitedChildren < startState.countChildren())) {

		bestAction = startState.getNextUnvisitedAction();
	}

	return bestAction;
}

double RolloutMonteCarlo::evaluate(GameState& state) {
	return 0.0; //Totally arbitrary, should be overriden by most subclasses
}

double RolloutMonteCarlo::evaluateFirst(GameState& parentState) {
	return DBL_MAX;
}

double RolloutMonteCarlo::estimateValue(GameState& state) {
	return state.searchData.empiricalValue;
}

void RolloutMonteCarlo::update(GameState& state, double playoutValue) {
	SearchData& data = state.searchData;

	//Update the rolling average
	data.empiricalValue =
			(data.empiricalValue * data.numVisits + playoutValue) /
			(data.numVisits + 1);

	data.numVisits++;
}

action_t RolloutMonteCarlo::currentBestMove() {
	GameState& root = mTree->getRoot();
	action_t numChildren = root.countChildren();
	action_t bestAction = 0;
	double bestValue = -DBL_MAX;

	bool isMinimizing = (root.getPlayer() == A);
	int minimaxNormalizer = isMinimizing ? -1 : 1;

	for(action_t c = 0; c < numChildren; c++) {
		double cValue = minimaxNormalizer * estimateValue(root.getChild(c));
		if(cValue > bestValue) {
			bestValue = cValue;
			bestAction = c;
		}
	}

	return bestAction;
}
