/*
 * UCTTuned.h
 *
 *  Created on: 9 apr 2010
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UCTTUNED_H_
#define UCTTUNED_H_

#include "UCTV.h"

/**
 * Represents an instance of the UCT-Tuned algorithm. While not strictly a derivative of
 * UCT-V (it's actually a derivative of UCT), the way it takes variance into account is
 * similar enough to warrant the sharing of code between UCT-V and UCT-Tuned.
 */
class UCTTuned: public UCTV {
public:

	/**
	 * Constructs a new instance of the UCT-V algorithm with the following parameters:
	 *
	 * @param horizon The depth of the tree to search down before evaluating instead of using
	 * UCT-V.
	 * @param discount The discount factor of the MDP.
	 * @param effectiveness The effectiveness of the algorithm's move simulation policy.
	 * @param fpuConst The FPU value of unexplored moves for this algorithm.
	 */
	UCTTuned(double effectiveness,
			double propogation,
			double fpu);

	virtual std::string toString() const;

	virtual ~UCTTuned();

protected:
	/**
	 * Returns the bias value of a given node with the UCT-Tuned policy. In this case, the bias
	 * of a node is computed using the estimated upper bound of the variance of the node's
	 * reward distribution.
	 *
	 * @param data The search data pertaining to the current node.
	 * @param parent The parent of the current node.
	 *
	 * @return The bias towards exploring this node.
	 */
	virtual double explorationBias(GameState& state);
};

#endif /* UCTTUNED_H_ */
