/*
 * UCTV.h
 *
 *  Created on: 7 avr. 2010
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef UCTV_H_
#define UCTV_H_

#include "UCT.h"

/**
 * Represents an instance of UCT-V, an extension to UCT. Rather than use the UCB1 move selection
 * policy, UCT-V uses the UCB-V move selection policy instead. This takes the variance of
 * payouts from nodes into account, such that internal nodes with high variance are given
 * higher priority for exploration.
 */
class UCTV: public UCT {
public:

	/**
	 * Constructs a new instance of the UCT-V algorithm with the following parameters:
	 *
	 * @param horizon The depth of the tree to search down before evaluating instead of using
	 * UCT-V.
	 * @param discount The discount factor of the MDP.
	 * @param effectiveness The effectiveness of the algorithm's move simulation policy.
	 */
	UCTV(	double effectiveness,
			double propogation,
			double fpu);
	~UCTV();

	/**
	 * Returns the bias value of a given node with the UCT-V policy. In this case, the bias
	 * of a node is computed using the variance and a nondecreasing sequence (of O(ln(ln t)))
	 * for t iterations.
	 *
	 * @param data The search data pertaining to the current node.
	 * @param parent The parent of the current node.
	 *
	 * @return The bias towards exploring this node.
	 */
	virtual double explorationBias(GameState& state);

	/**
	 * Updates the state of the node with the new mean and variance.
	 *
	 * @param data The search data pertaining to the current node.
	 * @param value The reward from choosing the given state.
	 */
	virtual void update(GameState& state, double value);

	virtual std::string toString() const;
};

#endif /* UCTV_H_ */
