/*
 * MinimaxGameState.cpp
 *
 *  Created on: Mar 22, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "MinimaxGameState.h"
#include "GameTreeGenerator.h"

MinimaxGameState::MinimaxGameState(
		GameTreeGenerator& generator,
		action_t numChildren,
		seed_t seed,
		player_t winner,
		double initialDifficulty) :
	GameState(generator, numChildren),
	mSeed(seed),
	mRng(seed),
	mDifficulty(initialDifficulty),
	mWinner(winner),
	mChildSeed(0U),
	mForcedChild(0U),
	mForcedWinner(NEITHER)
{ init(); }

MinimaxGameState::MinimaxGameState(
		GameTreeGenerator& generator,
		GameState& parent,
		action_t numChildren,
		seed_t seed,
		player_t winner,
		double parentDifficulty) :
	GameState(generator, parent, numChildren),
	mSeed(seed),
	mRng(seed),
	mDifficulty(mRng.nextVaried(parentDifficulty)),
	mWinner(winner),
	mChildSeed(0U),
	mForcedChild(0U),
	mForcedWinner(NEITHER)
{ init(); }

void MinimaxGameState::init() {
	//If the parent didn't force a winner on us, choose our own randomly based on
	// that weight
	if(mWinner == NEITHER) {
		mWinner = mRng.nextPlayer(mDifficulty);
	}

	//The rest deals with children, so if we don't have any, don't bother...
	if(isTerminal()) return;

	//Determine the seed of the first child
	mChildSeed = mRng.nextSeed();

	//Determine how we need to be forcing children to win states
	if(mWinner == getPlayer()) {
		//at least one child must be mWinner
		mForcedChild = mRng.nextAction(countChildren());
	} else {
		//All children must be mWinner
		mForcedWinner = mWinner;
	}
}

MinimaxGameState* MinimaxGameState::generateChild(action_t index) {
	seed_t childSeed = mRng.nthSeed(mChildSeed, index);

	action_t numGrandchildren;
	int overDepth = getDepth() - getTreeGenerator().getMaxDepth() + 1;
	if(overDepth >= 0 || (overDepth == -1 && ((childSeed >> 15) & 1) == 0)) {
		numGrandchildren = 0;
	} else {
		numGrandchildren = countChildren();
	}

	player_t winner = mForcedWinner;
	if(winner == NEITHER && index == mForcedChild) {
		winner = mWinner;
	}

	void* memoryLoc = getTreeGenerator().nextState();
	MinimaxGameState* next = new(memoryLoc) MinimaxGameState(
			getTreeGenerator(),
			*this,
			numGrandchildren,
			childSeed,
			winner,
			mDifficulty);
	return next;

}

player_t MinimaxGameState::getWinner() const {
	return mWinner;
}

double MinimaxGameState::getDifficulty() const {
	return mDifficulty;
}

double MinimaxGameState::getWinProbability() const {
	// Magic algebra, ahoy!
	// Because child difficulty is randomly varied, and because they're all
	//  randomly generated in the end anyway, this won't jive EXACTLY with
	//  generated children.
	// But it should be a pretty accurate probability.
	player_t currentPlayer = getPlayer();
	player_t winningPlayer = mWinner;

	double d = mDifficulty;
	double b = countChildren();

	if(currentPlayer == A && winningPlayer == B) return 1;
	if(currentPlayer == B && winningPlayer == A) return 0;

	//Magic algebra, ahoy!
	double bd = b*d;
	double bProb = (b + b*bd - bd) / (b*b - bd + bd*d - d*d + d);

	if(currentPlayer == A) {
		return bProb * (d*(b-1) / b);
	}

	// currentPlayer == B
	return bProb;
}

seed_t MinimaxGameState::getSeed() const {
	return mSeed;
}
