/*
 * RandomGenerator.cpp
 *
 *  Created on: Mar 23, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "RandomGenerator.h"
#include <float.h>

RandomGenerator::RandomGenerator() :
	mState(time(NULL))
{ }

RandomGenerator::RandomGenerator(seed_t startingSeed) :
	mState(startingSeed)
{ }

RandomGenerator::~RandomGenerator() {
}

double RandomGenerator::nextReal() {
	return ((double) next<unsigned int>()) / UINT_MAX;
}

inline double RandomGenerator::nextNormal() {
	double p = nextReal();

	static double a1 = -3.969683028665376e+01;
	static double a2 =  2.209460984245205e+02;
	static double a3 = -2.759285104469687e+02;
	static double a4 =  1.383577518672690e+02;
	static double a5 = -3.066479806614716e+01;
	static double a6 =  2.506628277459239e+00;

	static double b1 = -5.447609879822406e+01;
	static double b2 =  1.615858368580409e+02;
	static double b3 = -1.556989798598866e+02;
	static double b4 =  6.680131188771972e+01;
	static double b5 = -1.328068155288572e+01;

	static double c1 = -7.784894002430293e-03;
	static double c2 = -3.223964580411365e-01;
	static double c3 = -2.400758277161838e+00;
	static double c4 = -2.549732539343734e+00;
	static double c5 =  4.374664141464968e+00;
	static double c6 =  2.938163982698783e+00;

	static double d1 =  7.784695709041462e-03;
	static double d2 =  3.224671290700398e-01;
	static double d3 =  2.445134137142996e+00;
	static double d4 =  3.754408661907416e+00;

	//Define break-points.

	static double p_low  = 0.02425;
	static double p_high = 1 - p_low;

	//Rational approximation for lower region.
	if(p <= 0) p = DBL_MIN;
	if(p >= 1) p = 1 - DBL_MIN;

	double q, r;

	if (p < p_low) {
		q = sqrt(-2*log(p));
		return (((((c1*q+c2)*q+c3)*q+c4)*q+c5)*q+c6) /
				((((d1*q+d2)*q+d3)*q+d4)*q+1);
	}

	//Rational approximation for central region.

	else if (p <= p_high) {
		q = p - 0.5;
		r = q*q;
		return (((((a1*r+a2)*r+a3)*r+a4)*r+a5)*r+a6)*q /
				(((((b1*r+b2)*r+b3)*r+b4)*r+b5)*r+1);
	}

	//Rational approximation for upper region.

	else {
		q = sqrt(-2*log(1-p));
		return -(((((c1*q+c2)*q+c3)*q+c4)*q+c5)*q+c6) /
				((((d1*q+d2)*q+d3)*q+d4)*q+1);
	}
}

#define LCG_A 1103515245
#define LCG_C 12345

template <typename t>
inline t RandomGenerator::next() {
	t value = 0;

	size_t numShorts = sizeof(t)/2;

	for(unsigned int i=0; i<numShorts; i++) {
		mState = ((seed_t) (mState * LCG_A)) + LCG_C;

		value <<= 16;
		value |= mState >> 16;
	}

	return value;
}


/**
 * Generates a new random action. If numActions remains constant,
 * this will be fastest. If numActions switches between a relatively low
 * number of values, this will be fast. Otherwise, no speedup from
 * doing it yourself.
 * @param numActions The total number of available actions
 * @return A randomly chosen action
 */
action_t RandomGenerator::nextAction(action_t numActions)
{
	return next<action_t>() % numActions;
}



void RandomGenerator::seed(seed_t seed)
{
	mState = seed;
}



double RandomGenerator::nextVaried(double original)
{
	double v = original + .1*nextNormal();

	return v;
}

#define MAX_BRANCHING 500
seed_t A_SEQ[MAX_BRANCHING];
seed_t C_SEQ[MAX_BRANCHING];
int i;

seed_t RandomGenerator::nthSeed(seed_t startSeed, action_t n) {
	if(A_SEQ[0]==0) {
		A_SEQ[0] = LCG_A;
		C_SEQ[0] = LCG_C;

		for(int i=1;i<MAX_BRANCHING;i++) {
			A_SEQ[i] = A_SEQ[i-1]*LCG_A;
			C_SEQ[i] = C_SEQ[i-1]+LCG_C*A_SEQ[i-1];
		}
	}

	return A_SEQ[n]*startSeed + C_SEQ[n];
}

seed_t RandomGenerator::nextSeed()
{
	return next<seed_t>();
}

player_t RandomGenerator::nextPlayer(double difficulty) {
	double r = nextReal();
	double p = 1.0/(1.0+exp(-difficulty));
	player_t player = (r < p) ? B : A;
	/*
	std::cout << std::endl <<
			"NP: seed=" << mState <<
			", probB=" << difficulty <<
			", nextR=" << r <<
			", plyr=" << player << std::endl;
	*/
	return player;
}
