/*
 * GombaMain.cpp
 *
 *  Created on: Mar 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <iostream>
#include <ostream>
#include <fstream>
#include <string>
#include <boost/program_options.hpp>
#include <boost/random/lagged_fibonacci.hpp>
#include "../gametree/GameTree.h"
#include "../gametree/GameTreeGenerator.h"
#include "../search/Searcher.h"
#include "../search/SearcherFactory.h"
#include "../gamedata/DataFormatter.h"

namespace po = boost::program_options;

int main(int argc, char* argv[]) {
	// Declare the supported options.
	po::options_description desc("Allowed options");
	desc.add_options()
	    ("help", "show help message")
	    ("seed,s", po::value<long>(), "set the random seed for tree generation [time in microsec]")
	    //("treetype,t", po::value<std::string>(), "set the tree generation algorithm")
	    ("trials,n",   po::value<unsigned int>(), "set the number of trials [1]")
	    ("depth,d",    po::value<unsigned int>(), "set the maximum tree depth [50]")
	    ("bias,B",     po::value<double>(), "set the game bias. Any decimal, 0 = even odds, lower = better for first player [0.0]")
	    ("branch,b",   po::value<action_t>(), "set the branching factor of the game [20]")
	    ("max-iterations,i", po::value<unsigned int>(), "set the maximum number of allowed iterations per move [10000]")
	    ("block-size,S", po::value<unsigned int>(), "set the memory block size for state pooling [1000]")
	    ("effectiveness,e", po::value< double >(), "set the effectiveness of move simulations")
	    ("searchtypes",po::value< std::vector <std::string> >(), "set the search type(s) to test [uct, mcs, random]")
	;

	po::positional_options_description p;
	p.add("searchtypes", -1);

	po::variables_map vm;
	po::store(po::command_line_parser(argc, argv).
	          options(desc).positional(p).run(), vm);
	po::notify(vm);

	if (vm.count("help")) {
	    std::cout << desc << std::endl;
	    return 1;
	}

	if(! vm.count("searchtypes")) {
		std::cout << "At least one search type must be specified" << std::endl;
		return 1;
	}

	std::vector< std::string > searchTypes = vm["searchtypes"].as<std::vector<std::string> >();

	seed_t seed;

	if (vm.count("seed")) {
		seed = vm["seed"].as<uint32_t>();
	} else {
		seed = ((seed_t) clock()) + ((seed_t) getpid());
	}

	if(vm.count("treetype")) {
		std::cout << "Unsupported, but nice try!" << std::endl;
		//TODO: support
	}

	unsigned int maxDepth = 50;
	if(vm.count("depth")) {
		maxDepth = vm["depth"].as<unsigned int>();
	}

	double bias = 0.0;
	if(vm.count("bias")) {
		bias = vm["bias"].as<double>();
	}

	action_t branchingFactor = 20;
	if(vm.count("branch")) {
		branchingFactor = vm["branch"].as<action_t>();
	}

	unsigned int trials = 1;
	if(vm.count("trials")) {
		trials = vm["trials"].as<unsigned int>();
	}

	unsigned int maxIterations = 1;
	if(vm.count("max-iterations")) {
		maxIterations = vm["max-iterations"].as<unsigned int>();
	}

	unsigned int blockSize = 1000;
	if(vm.count("block-size")) {
		blockSize = vm["block-size"].as<unsigned int>();
	}

	double effectiveness = 0.5;
	if(vm.count("effectiveness")) {
		effectiveness = vm["effectiveness"].as<double>();
	}

	GameTreeGenerator treeGenerator(branchingFactor, maxDepth, bias, seed, blockSize);

	SearcherFactory searcherFactory;
	std::vector<Searcher> & searchers =
			searcherFactory.generateSearchers(
					searchTypes,
					maxIterations,
					effectiveness);

	for(unsigned int trial=0 ; trial < trials ; trial++) {
		GameTree* tree = treeGenerator.generate();
		for(unsigned int sid = 0; sid < searchers.size(); sid++) {
			searchers[sid].play(*tree);
			tree->reset();
		}
		delete tree;
	}

	for(unsigned int sid = 0; sid < searchers.size(); sid++) {
		ofstream outfile((searchers[sid].getIdentifier() + ".dat").c_str());
		DataFormatter formatter(outfile);
		formatter.parse(searchers[sid]);
		outfile.close();
	}

}
