/*
 * SearchAlgorithm.h
 *
 *  Created on: 2010.03.19.
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SEARCHALGORITHM_H_
#define SEARCHALGORITHM_H_

#include <string>
#include <sstream>
#include "../gametree/GameTree.h"
#include "../gametree/Action.h"

/**
 * Defines an abstract search algorithm to run on a tree. Search algorithms must implement
 * all of the named methods as specified in each method description.
 */
class SearchAlgorithm {
protected:
	/**
	 * This is the tree that the individual algorithms should be using as input. The
	 * generalized SearchAlgorithm methods deal with keeping it up to date across reset()s
	 *
	 * Subclasses should treat this variable as const GameTree* const
	 */
	const GameTree* mTree;

	/**
	 * Resets the internal state of this algorithm. The public reset(GameTree&) method
	 * will call this internal one.
	 */
	virtual void reset() = 0;
public:

	/**
	 * Run one "iteration" of this algorithm. In practice, this will be run probably
	 * several thousand times before a useful result is needed from currentBestMove().
	 */
	virtual void iterate() = 0;

	/**
	 * Use the information gathered by previous calls to iterate() to determine which
	 * action should next be played.
	 *
	 * @return The index of the best move among the children of the current game state.
	 */
	virtual action_t currentBestMove() = 0;

	/**
	 * Return the name of the search algorithm for comparison purposes. There is no specified
	 * naming convention.
	 *
	 * @return The name of the algorithm.
	 */
	virtual std::string toString() const = 0;

	/**
	 * Resets the algorithm back to an initial state. This method must erase any and all
	 * internal state that the algorithm was tracking, so as to ensure repeatability of results
	 * should multiple trials be desirable with the same algorithm object.
	 *
	 * This method also tells the algorithm which tree subsequent iterate() and best() results
	 * should be based on
	 */
	virtual void reset(const GameTree& tree) {
		mTree = &tree;
		reset();
	}

};

#endif /* SEARCHALGORITHM_H_ */
