/*
 * Searcher.h
 *
 *  Created on: 2010.03.19.
 *      Author: jnschaeffer
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SEARCHER_H_
#define SEARCHER_H_

#include "SearchAlgorithm.h"
#include <time.h>
#include "../gametree/GameTree.h"
#include <list>
#include <string>

struct ResultData {
	unsigned int wins;
	double totalDifficulty;
	clock_t totalDuration;
	unsigned long int nodesExpanded;
};

/**
 * Represents a searcher to run different search algorithms against one another
 * (or themselves). A Searcher takes one or two search algorithms and can play them
 * on any given game tree.
 */
class Searcher {

public:

	/**
	 * Construct a Searcher with a single search algorithm that will play against itself.
	 *
	 * @param alg The algorithm to be used. Will not be deleted upon destruction of Searcher.
	 * @param max Maximum number of iterations permitted.
	 */
	Searcher(std::string name, SearchAlgorithm& alg, unsigned int max);

	/**
	 * Play a single game on the given game tree. This will play the game, but will not
	 * reset the tree to any starting state.
	 *
	 *
	 * @return Reference to the game's data. This will NOT be freed on the destruction of
	 * Searcher.
	 */
	void play(GameTree& tree);

	/**
	 * @return A vector of win counts for the given player at each iteration of the algorithm in
	 *         all games at all move steps. This vector will be destroyed with the Searcher.
	 */
	std::vector<ResultData>& getResults();

	std::string getIdentifier();

	~Searcher();

private:
	std::string mName;
	SearchAlgorithm* mAlgorithm;
	int mMaxIterations;
	std::vector<ResultData> mResults;

};

#endif /* SEARCHER_H_ */
