/*
 * SearcherFactory.h
 *
 *  Created on: Mar 19, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SEARCHERFACTORY_H_
#define SEARCHERFACTORY_H_

#include "Searcher.h"
#include "SearchAlgorithm.h"
#include "../gametree/RandomGenerator.h"
#include <vector>
#include <list>
#include <map>

using namespace std;

/**
 * A simple factory class for generating search algorithms to run on trees. Any Searchers that
 * this factory generates will last until they are destroyed by other classes outside of the
 * SearcherFactory. However, the algorithms underpinning the Searcher will only last the length
 * of the SearcherFactory. Thus, while Searcher objects will last as long as is needed, without
 * being tracked by the SearcherFactory, they should not be used for most purposes when the
 * SearcherFactory is destroyed.
 *
 */
class SearcherFactory {

public:
	SearcherFactory();
	~SearcherFactory();

	/**
	 * Generates as many searchers as there are algorithms to generate searchers for, with
	 * an upper limit on the number of iterations algorithms may use. The search algorithms
	 * will last the lifetime of the SearcherFactory, *not* the SearchAlgorithm.
	 *
	 * @param algs The names of algorithms to generate searchers for.
	 * @param max  The maximum number of iterations the search algorithms can run for.
	 * @param effectiveness The effectiveness of the algorithms' simulation policies. Not
	 *                      necessarily used for all algorithms.
	 *
	 * @return The vector of searchers. This will NOT be freed by SearcherFactory.
	 */
	vector<Searcher>& generateSearchers(vector<string>& algs,
										unsigned int max,
										double effectiveness);

	/**
	 * Generates a SearchAlgorithm based on a string name.
	 *
	 * @param name The name of the algorithm to generate.
	 */
	SearchAlgorithm& generateAlgorithm(string name, double effectiveness);

private:

	// Previously-allocated SearchAlgorithms.
	list<SearchAlgorithm*> allocatedAlgs;

};

#endif /* SEARCHERFACTORY_H_ */
