/*
 * RandomGeneratorTest.cpp
 *
 *  Created on: Mar 29, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "../gametree/RandomGenerator.h"
#include <gtest/gtest.h>

namespace {

class RandomGeneratorTest : public ::testing::Test {
protected:
	RandomGenerator rng;
};

TEST_F(RandomGeneratorTest, SeedUsuallyVaries) {
	unsigned int timesChanged = 0;
	unsigned int iterations = 1000;

	seed_t curSeed = 1;
	seed_t prevSeed = 0;

	rng.seed(curSeed);

	for(unsigned int i=0;i<iterations;i++) {
		prevSeed = curSeed;
		curSeed = rng.nextSeed();
		if(prevSeed != curSeed) timesChanged++;
	}

	EXPECT_GE(timesChanged, iterations * 0.95);
}

TEST_F(RandomGeneratorTest, RealDistribution) {
	int iterations = 10000;
	double total = 0;
	double totalVar = 0;
	for(int i=0;i<iterations;i++) {
		double r = rng.nextReal();
		ASSERT_LE(r, 1.0);
		ASSERT_GE(r, 0.0);
		total += r;
		totalVar += (r > 0.5) ? r - 0.5 : 0.5 - r;
	}
	double avg = total / iterations;
	double avgVar = totalVar / iterations;
	EXPECT_GE(avg, 0.49);
	EXPECT_LE(avg, 0.51);
	EXPECT_GE(avgVar, 0.24);
	EXPECT_LE(avgVar, 0.26);
}

TEST_F(RandomGeneratorTest, WinnerForNearSeeds) {
	int iterations = 10000;
	int flips = 0;

	seed_t seed = 1234;
	player_t lastWinner = A;

	for(int i = 0 ; i < iterations ; i++) {
		seed++;
		rng.seed(seed);
		player_t winner = rng.nextPlayer(0.0);
		if(winner != lastWinner) flips++;
	}

	EXPECT_GE(flips, .49*iterations);
	EXPECT_LE(flips, .51*iterations);
}

TEST_F(RandomGeneratorTest, RealDistribution2) {
	int iterations = 10000;
	int flips = 0;
	bool aboveHalf = false;
	for(int i=0;i<iterations;i++) {
		double d = rng.nextReal();
		bool a = d > 0.5;
		if(a != aboveHalf) flips++;
		aboveHalf = a;
	}
	EXPECT_GE(flips, .49*iterations);
	EXPECT_LE(flips, .51*iterations);
}


//Tests that the varied value is on average near the expected input center
TEST_F(RandomGeneratorTest, VariedCenteredNearArg) {
	int iterations = 1000;

	for(double center=0.0 ; center < 1.0; center += .03) {
		double total = 0;
		for(int i = 0; i < iterations ; i++) {
			total += rng.nextVaried(center);
		}
		double avg = total / iterations;

		EXPECT_LE(avg, center+0.1);
		EXPECT_GE(avg, center-0.1);
	}
}

//Tests that the varied value is in fact typically different from the center
TEST_F(RandomGeneratorTest, VariedActuallyVaries) {
	int iterations = 1000;

	for(double center=0.0 ; center < 1.0; center += 0.027) {
		double totalVariance = 0;
		for(int i = 0; i < iterations ; i++) {
			double diff = rng.nextVaried(center) - center;
			if(diff < 0) diff *= -1;
			totalVariance += diff;
		}
		double avg = totalVariance / iterations;

		EXPECT_GE(avg, 0.001);
		EXPECT_LE(avg, 0.500);
	}
}

}
