/*
 * TreeTest.cpp
 *
 *  Created on: Mar 29, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "../gametree/GameTreeGenerator.h"
#include <gtest/gtest.h>
#include <set>

namespace {

class TreeTest : public ::testing::Test {
};

TEST_F(TreeTest, RootNodeWithinParameters) {
	GameTreeGenerator treeGenerator(10, 10, .5, time(NULL), 1000);
	GameTree* tree = treeGenerator.generate();
	ASSERT_FALSE(tree == NULL);
	EXPECT_EQ(tree->getRoot().countChildren(), 10);
}

TEST_F(TreeTest, ManyChildrenRightDepthAndBranching) {
	unsigned short branching = 4;
	unsigned int depth = 12;

	GameTreeGenerator treeGenerator(branching, depth, .5, time(NULL), 1000);
	GameTree* tree;

	for(int i=0;i<100000;i++) {
		tree = treeGenerator.generate();
		ASSERT_TRUE(tree != NULL);
		GameState* state = &(tree->getRoot());
		unsigned int d;
		for(d=0;d<depth;d++) {
			ASSERT_TRUE(state != NULL);
			EXPECT_EQ(d, state->getDepth());
			if(state->isTerminal()) break;
			ASSERT_EQ(branching, state->countChildren());
			state = &(state->getChild(d % branching));
		}
		ASSERT_TRUE(state != NULL);
		EXPECT_EQ(d, state->getDepth());
		EXPECT_EQ(0, state->countChildren());
		EXPECT_EQ(true, state->isTerminal());
	}
}

void testTree(GameState& state, std::set<seed_t>& seedSet, unsigned int* repeats) {
	seed_t s = state.getSeed();
	if(seedSet.find(s) != seedSet.end()) {
		std::cout << "!";
		(*repeats)++;
	} else {
		seedSet.insert(s);
	}

	for(int i=0;i<state.countChildren();i++) {
		testTree(state.getChild(i), seedSet, repeats);
	}
}

TEST_F(TreeTest, IndependentNodeGenerationOrder) {
	const unsigned int PATH_LENGTH = 5;
	const unsigned int NUM_PATHS   = 3;
	action_t paths[NUM_PATHS][PATH_LENGTH] =
		{
				{ 2, 6,39,21, 3},
				{25, 3,42, 1,22},
				{ 1, 2, 3, 4, 5}
		};
	seed_t seeds[NUM_PATHS];

	unsigned short branching = 50;
	unsigned int depth = 10;

	GameTreeGenerator treeGenerator(branching, depth, 0, time(NULL), 1000);
	GameTree* tree = treeGenerator.generate();
	GameState& root = tree->getRoot();

	for(unsigned int i=0;i<NUM_PATHS;i++) {
		GameState* state = &root;
		for(unsigned int d=0;d<PATH_LENGTH;d++) {
			state = &state->getChild(paths[i][d]);
		}
		seeds[i] = state->getSeed();
		state->searchData.customInt = 27;
	}

	tree->reset();

	for(unsigned int i=0;i<NUM_PATHS;i++) {
		GameState* state = &root;
		for(unsigned int d=0;d<PATH_LENGTH;d++) {
			state = &state->getChild(paths[NUM_PATHS - i - 1][d]);
		}
		ASSERT_EQ(seeds[NUM_PATHS - i - 1], state->getSeed());
		ASSERT_NE(state->searchData.customInt, 27);
	}
}

TEST_F(TreeTest, IndependentNodes) {
	//We will do a full depth first search.
	//Make sure branching^(depth+1)-1 is reasonable.
	unsigned short branching = 2;
	unsigned int depth = 14;

	std::set<seed_t> usedSeeds;
	unsigned int repeats = 0;
	GameTreeGenerator treeGenerator(branching, depth, 0, time(NULL), 1000);
	GameTree* tree = treeGenerator.generate();
	GameState& root = tree->getRoot();

	testTree(root, usedSeeds, &repeats);

	ASSERT_LE(repeats, 10);

}

}
