/*
 * treeexplorer.cpp
 *
 *  Created on: Apr 2, 2010
 *      Author: dbjorge
 *
 * Copyright 2010 Worcester Polytechnic Institute
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string>
#include <time.h>
#include <unistd.h>
#include "../gametree/GameTreeGenerator.h"
#include "../search/SearchAlgorithm.h"
#include "../search/SearcherFactory.h"

using namespace std;

void printState(GameState& state) {
	cout << &state << " :: "
			<< "Wins:" << state.getWinner()
			<< " Diff:" << state.getDifficulty()
			<< " Depth:" << state.getDepth()
			<< " #Kids:" << state.countChildren()
			<< " Parent:" << state.getParent()
			<< " Est: " << state.searchData.estimatedValue
			<< " Emp: " << state.searchData.empiricalValue
			<< " Visits: " << state.searchData.numVisits
			<< endl;
}

int main(int argc, char* argv[]) {
	action_t branching;
	unsigned int depth;
	double bias = 0;
	double effectiveness = 0.05;
	seed_t seed = clock() + getpid();
	unsigned int blockSize = 1000;

	branching = atoi(argv[1]);
	depth = atoi(argv[2]);
	bias = atof(argv[3]);

	GameTreeGenerator treeGenerator(branching, depth, bias, seed, blockSize);

	cout << "New tree starting..." << endl;
	GameTree* tree = treeGenerator.generate();
	GameState* current = &(tree->getRoot());

	SearcherFactory factory;
	SearchAlgorithm* alg = &factory.generateAlgorithm("random", effectiveness);
	unsigned int iteration = 0;

	cout << "Instructions:" << endl
			<< "\tq to quit" << endl
			<< "\tnumbers to go to children" << endl
			<< "\tp to go to parent" << endl
			<< "\tc to list all children" << endl
			<< "\treset to reset the tree and return to the root" << endl
			<< "\ts<num> to run num monte-carlo simulations" << endl
			<< "\ta<algname> to choose a new search algorithm" << endl
			<< "\ti<num> to iterate through the search algorithm num times" << endl
			<< "\tb to ask the algorithm to guess" << endl
			<< "...An invalid command will be interpreted as a 0." << endl;

	RandomGenerator rng;
	string command = "";
	while(command != "q") {
		printState(*current);
		cout << "Using algorithm " << alg->toString() << " (iteration " << iteration << ")" << endl;

		cin >> command;

		if(command == "p") {
			current = current->getParent();
		} else if(command == "q") {
			break;
		} else if(command == "reset") {
			current = &tree->getRoot();
			tree->reset();
			iteration = 0;
			alg->reset(*tree);
		} else if(command == "c") {
			for(int i=0;i<current->countChildren();i++) {
				cout << i << " = ";
				printState(current->getChild(i));
			}
		} else if(command.substr(0,1)=="a") {
			delete alg;
			current = &tree->getRoot();
			tree->reset();
			alg = &factory.generateAlgorithm(command.substr(1, command.size()-1), effectiveness);
			alg->reset(*tree);
			iteration = 0;

		} else if(command.substr(0,1)=="i") {
			int its = atoi(command.substr(1, command.size()-1).c_str());
			for(int i=0;i<its;i++) {
				iteration++;
				alg->iterate();
			}
			cout << "Iterated " << its << " times" << endl;
		} else if(command == "b") {
			action_t best = alg->currentBestMove();
			cout << "Best action: " << best << endl;
			cout << best << " :: ";
			printState(tree->getRoot().getChild(best));
		} else if(command.substr(0,1)=="s") {
			GameState* c;
			unsigned int Awins = 0;
			int iterations = atoi(command.substr(1, command.size()-1).c_str());
			for(int i=0;i<iterations;i++) {
				c = current;
				while(!c->isTerminal()) c = &c->getChild(rng.nextAction(c->countChildren()));
				//cout << "Simulated winner: " << c->getWinner() << endl;
				if(c->getWinner() == current->getPlayer()) {
					Awins++;
				}
			}
			cout << "Actual win probability for current player: " << (((double) Awins) / iterations) << endl;
		} else {
			current = &current->getChild(atoi(command.c_str()));
		}
	}

	delete tree;
}
